!-------------------------------------------------------------------------------

!                      Code_Saturne version 4.0.0
!                      --------------------------
! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2015 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------

!===============================================================================
! Purpose:
! -------

!> \file cs_user_extra_operations.f90
!>
!> \brief This function is called at the end of each time step, and has a very
!>  general purpose
!>  (i.e. anything that does not have another dedicated user subroutine)
!>
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]     nvar          total number of variables
!> \param[in]     nscal         total number of scalars
!> \param[in]     dt            time step (per cell)
!_______________________________________________________________________________

subroutine cs_f_user_extra_operations &
 ( nvar   , nscal  ,                                              &
   dt     )

!===============================================================================

!===============================================================================
! Module files
!===============================================================================

use paramx
use dimens, only: ndimfb
use pointe
use numvar
use optcal
use cstphy
use cstnum
use entsor
use lagpar
use lagran
use lagdim
use parall
use period
use ppppar
use ppthch
use ppincl
use mesh
use field
use field_operator
use turbomachinery
use cs_c_bindings

!===============================================================================

implicit none

! Arguments

integer          nvar   , nscal

double precision dt(ncelet)

! Local variables
integer impout
integer npoint, ii, nlelt, ilelt, iel, ifac
integer, dimension(:), allocatable :: lstelt
double precision xyz(3), zmin, zmax, zplane, deltaz, zunder, zover, velarray
double precision totvol, velave, tempave
 character*35 condition
double precision, dimension(:), pointer :: temparray
double precision, dimension(:,:), pointer :: cvar_vel
!===============================================================================

!===============================================================================
! Initialization
!===============================================================================

! INSERT_ADDITIONAL_INITIALIZATION_CODE_HERE

!===============================================================================
! User operations
!===============================================================================


!BULK VEL AND TEMP CALC.
if (ntcabs.eq.ntmabs) then ! only at last timestep and only 
    write (nfecra,*) 'initialize the array...'
    allocate(lstelt(ncel))  ! if the thermal model involves the Temperature
    call field_get_val_v(ivarfl(iu), cvar_vel)
    call field_get_val_s(ivarfl(isca(iscalt)), temparray)
    impout = impusr(1)
    
    if (irangp.le.0) then
        open(impout, file='Bulk_UT.dat')
        write(impout,*) '# z(m) U(m/s) T(K)'
    endif

    npoint = 200
 !   iel = -999
 !  irang = -999
    zmin = 0
    zmax = 3.92
    deltaz = (zmax - zmin)/npoint
    
    write (nfecra,*) 'Start bulk calculation...'
    
    do ii = 1, npoint
      totvol = 0
      tempave = 0
      velave = 0
      zplane = zmin + (float(ii-1)*(zmax - zmin)/float(npoint-1))
      zunder = zplane - 0.5*deltaz
      zover = zplane + 0.5*deltaz
      write (condition, "(A5, BN, F10.2, A10, BN, F10.2)") 'z >= ', zunder, ' and z <= ', zover
      call getcel(condition, nlelt, lstelt)
      if (nlelt.gt.0) then
        write (nfecra,*) 'calculate volume and total parameters...'
        do ilelt = 1, nlelt
          ifac=lstelt(ilelt)
          iel= ifabor(ifac)
          velarray=cvar_vel(3, iel)
          totvol = totvol + volume(lstelt(ilelt))
          tempave = tempave + volume(lstelt(ilelt))*temparray(lstelt(ilelt))
          velave = velave + volume(lstelt(ilelt))*velarray
        end do
        if (irangp.ge.0) then
          write (nfecra,*) 'call sum of parameters...'
          call parsom(velave)
          write (nfecra,*) 'velave called...'
          call parsom(tempave)
          write (nfecra,*) 'tempave called...'
          call parsom(totvol)
          write (nfecra,*) 'totvol called...'
        endif
        tempave = tempave/totvol
        velave = velave/totvol
        if (irangp.le.0) then
          write (nfecra,*) 'write to file...'
          write (impout,'(F10.4, F10.4, F10.4)') zplane, velave, tempave
        endif        
      endif
    end do 
    if (irangp.le.0) close(impout)
deallocate(lstelt)
end if

!--------
! Formats
!--------

!----
! End
!----


return
end subroutine cs_f_user_extra_operations
